@@ -42,15 +42,6 @@ func NewMakeCommands() *cobra.Command {
 		Use:   "build",
 		Short: "Build delve",
 		Run: func(cmd *cobra.Command, args []string) {
-			tagFlag := prepareMacnative()
-			if len(*Tags) > 0 {
-				if len(tagFlag) == 0 {
-					tagFlag = "-tags="
-				} else {
-					tagFlag += ","
-				}
-				tagFlag += strings.Join(*Tags, ",")
-			}
 			envflags := []string{}
 			if len(Architecture) > 0 {
 				envflags = append(envflags, "GOARCH="+Architecture)
@@ -59,9 +50,9 @@ func NewMakeCommands() *cobra.Command {
 				envflags = append(envflags, "GOOS="+OS)
 			}
 			if len(envflags) > 0 {
-				executeEnv(envflags, "go", "build", "-ldflags", "-extldflags -static", tagFlag, buildFlags(), DelveMainPackagePath)
+				executeEnv(envflags, "go", "build", "-ldflags", "-extldflags -static", tagFlags(), buildFlags(), DelveMainPackagePath)
 			} else {
-				execute("go", "build", "-ldflags", "-extldflags -static", tagFlag, buildFlags(), DelveMainPackagePath)
+				execute("go", "build", "-ldflags", "-extldflags -static", tagFlags(), buildFlags(), DelveMainPackagePath)
 			}
 			if runtime.GOOS == "darwin" && os.Getenv("CERT") != "" && canMacnative() {
 				codesign("./dlv")
@@ -78,8 +69,7 @@ func NewMakeCommands() *cobra.Command {
 		Use:   "install",
 		Short: "Installs delve",
 		Run: func(cmd *cobra.Command, args []string) {
-			tagFlag := prepareMacnative()
-			execute("go", "install", tagFlag, buildFlags(), DelveMainPackagePath)
+			execute("go", "install", tagFlags(), buildFlags(), DelveMainPackagePath)
 			if runtime.GOOS == "darwin" && os.Getenv("CERT") != "" && canMacnative() {
 				codesign(installedExecutablePath())
 			}
@@ -271,7 +261,8 @@ func canMacnative() bool {
 
 	typesHeader := "/usr/include/sys/types.h"
 	if major >= 11 || (major == 10 && minor >= 15) {
-		typesHeader = "/Library/Developer/CommandLineTools/SDKs/MacOSX.sdk/usr/include/sys/types.h"
+		sdkpath := strings.TrimSpace(getoutput("xcrun", "--sdk", "macosx", "--show-sdk-path"))
+		typesHeader = filepath.Join(sdkpath, "usr", "include", "sys", "types.h")
 	}
 	_, err = os.Stat(typesHeader)
 	if err != nil {
@@ -289,15 +280,34 @@ func prepareMacnative() string {
 	if !checkCert() {
 		return ""
 	}
-	return "-tags=macnative"
+	return "macnative"
+}
+
+func tagFlags() string {
+	var tags []string
+	if mactags := prepareMacnative(); mactags != "" {
+		tags = append(tags, mactags)
+	}
+	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
+		tags = append(tags, "exp.winarm64")
+	}
+	if Tags != nil && len(*Tags) > 0 {
+		tags = append(tags, *Tags...)
+	}
+	if len(tags) == 0 {
+		return ""
+	}
+	return "-tags=" + strings.Join(tags, ",")
 }
 
 func buildFlags() []string {
+	var ldFlags string
 	buildSHA, err := getBuildSHA()
 	if err != nil {
-		log.Fatal(fmt.Errorf("error getting build SHA via git: %w", err))
+		log.Printf("error getting build SHA via git: %w", err)
+	} else {
+		ldFlags = "-X main.Build=" + buildSHA
 	}
-	ldFlags := "-X main.Build=" + buildSHA
 	if runtime.GOOS == "darwin" {
 		ldFlags = "-s " + ldFlags
 	}
@@ -387,8 +397,9 @@ func testStandard() {
 		case "linux":
 			dopie = true
 		case "windows":
+			// windows/arm64 always uses pie buildmode, no need to test everything again.
 			// only on Go 1.15 or later, with CGO_ENABLED and gcc found in path
-			if goversion.VersionAfterOrEqual(runtime.Version(), 1, 15) {
+			if runtime.GOARCH != "arm64" && goversion.VersionAfterOrEqual(runtime.Version(), 1, 15) {
 				out, err := exec.Command("go", "env", "CGO_ENABLED").CombinedOutput()
 				if err != nil {
 					panic(err)
@@ -442,13 +453,17 @@ func testCmdIntl(testSet, testRegex, testBackend, testBuildMode string) {
 		buildModeFlag = "-test-buildmode=" + testBuildMode
 	}
 
+	env := []string{}
+	if os.Getenv("CI") != "" {
+		env = os.Environ()
+	}
+
 	if len(testPackages) > 3 {
-		env := []string{}
-		executeq(env, "go", "test", testFlags(), buildFlags(), testPackages, backendFlag, buildModeFlag)
+		executeq(env, "go", "test", testFlags(), buildFlags(), tagFlags(), testPackages, backendFlag, buildModeFlag)
 	} else if testRegex != "" {
-		execute("go", "test", testFlags(), buildFlags(), testPackages, "-run="+testRegex, backendFlag, buildModeFlag)
+		executeq(env, "go", "test", testFlags(), buildFlags(), tagFlags(), testPackages, "-run="+testRegex, backendFlag, buildModeFlag)
 	} else {
-		execute("go", "test", testFlags(), buildFlags(), testPackages, backendFlag, buildModeFlag)
+		executeq(env, "go", "test", testFlags(), buildFlags(), tagFlags(), testPackages, backendFlag, buildModeFlag)
 	}
 }
 
@@ -487,7 +502,7 @@ func inpath(exe string) bool {
 
 func allPackages() []string {
 	r := []string{}
-	for _, dir := range strings.Split(getoutput("go", "list", "-mod=vendor", "./..."), "\n") {
+	for _, dir := range strings.Split(getoutput("go", "list", "-mod=vendor", tagFlags(), "./..."), "\n") {
 		dir = strings.TrimSpace(dir)
 		if dir == "" || strings.Contains(dir, "/vendor/") || strings.Contains(dir, "/_scripts") {
 			continue
