@@ -305,7 +305,7 @@ only see the output of the trace operations you can redirect stdout.`,
 	traceCommand.Flags().StringVarP(&traceExecFile, "exec", "e", "", "Binary file to exec and trace.")
 	traceCommand.Flags().BoolVarP(&traceTestBinary, "test", "t", false, "Trace a test binary.")
 	traceCommand.Flags().BoolVarP(&traceUseEBPF, "ebpf", "", false, "Trace using eBPF (experimental).")
-	traceCommand.Flags().IntVarP(&traceStackDepth, "stack", "s", 0, "Show stack trace with given depth. (Ignored with -ebpf)")
+	traceCommand.Flags().IntVarP(&traceStackDepth, "stack", "s", 0, "Show stack trace with given depth. (Ignored with --ebpf)")
 	traceCommand.Flags().String("output", "debug", "Output path for the binary.")
 	rootCommand.AddCommand(traceCommand)
 
@@ -327,6 +327,10 @@ Currently supports linux/amd64 and linux/arm64 core files, windows/amd64 minidum
 		},
 		Run: coreCmd,
 	}
+	// -c is unused and exists so delve can be used with coredumpctl
+	core := false
+	coreCommand.Flags().BoolVarP(&core, "core", "c", false, "")
+	coreCommand.Flags().MarkHidden("core")
 	rootCommand.AddCommand(coreCommand)
 
 	// 'version' subcommand.
@@ -480,6 +484,7 @@ func dapCmd(cmd *cobra.Command, args []string) {
 				Foreground:           true, // server always runs without terminal client
 				DebugInfoDirectories: conf.DebugInfoDirectories,
 				CheckGoVersion:       checkGoVersion,
+				DisableASLR:          disableASLR,
 			},
 			CheckLocalConnUser: checkLocalConnUser,
 		}
@@ -585,7 +590,7 @@ func traceCmd(cmd *cobra.Command, args []string) {
 		var debugname string
 		if traceAttachPid == 0 {
 			if dlvArgsLen >= 2 && traceExecFile != "" {
-				fmt.Fprintln(os.Stderr, "Cannot specify package when using exec.")
+				fmt.Fprintln(os.Stderr, "Cannot specify package when using --exec.")
 				return 1
 			}
 
@@ -627,16 +632,20 @@ func traceCmd(cmd *cobra.Command, args []string) {
 			return 1
 		}
 		client := rpc2.NewClientFromConn(clientConn)
+		defer client.Detach(true)
 		funcs, err := client.ListFunctions(regexp)
 		if err != nil {
 			fmt.Fprintln(os.Stderr, err)
 			return 1
 		}
+		success := false
 		for i := range funcs {
 			if traceUseEBPF {
 				err := client.CreateEBPFTracepoint(funcs[i])
 				if err != nil {
 					fmt.Fprintf(os.Stderr, "unable to set tracepoint on function %s: %#v\n", funcs[i], err)
+				} else {
+					success = true
 				}
 			} else {
 				// Fall back to breakpoint based tracing if we get an error.
@@ -649,10 +658,14 @@ func traceCmd(cmd *cobra.Command, args []string) {
 				})
 				if err != nil && !isBreakpointExistsErr(err) {
 					fmt.Fprintf(os.Stderr, "unable to set tracepoint on function %s: %#v\n", funcs[i], err)
+					continue
+				} else {
+					success = true
 				}
 				addrs, err := client.FunctionReturnLocations(funcs[i])
 				if err != nil {
 					fmt.Fprintf(os.Stderr, "unable to set tracepoint on function %s: %#v\n", funcs[i], err)
+					continue
 				}
 				for i := range addrs {
 					_, err = client.CreateBreakpoint(&api.Breakpoint{
@@ -664,19 +677,25 @@ func traceCmd(cmd *cobra.Command, args []string) {
 					})
 					if err != nil && !isBreakpointExistsErr(err) {
 						fmt.Fprintf(os.Stderr, "unable to set tracepoint on function %s: %#v\n", funcs[i], err)
+					} else {
+						success = true
 					}
 				}
 			}
 		}
+		if !success {
+			fmt.Fprintln(os.Stderr, "no breakpoints set")
+			return 1
+		}
 		cmds := terminal.DebugCommands(client)
 		t := terminal.New(client, nil)
+		t.SetTraceNonInteractive()
 		t.RedirectTo(os.Stderr)
 		defer t.Close()
 		if traceUseEBPF {
 			done := make(chan struct{})
 			defer close(done)
 			go func() {
-				gFnEntrySeen := map[int]struct{}{}
 				for {
 					select {
 					case <-done:
@@ -698,14 +717,11 @@ func traceCmd(cmd *cobra.Command, args []string) {
 									params.WriteString(p.Value)
 								}
 							}
-							_, seen := gFnEntrySeen[t.GoroutineID]
-							if seen {
+							if t.IsRet {
 								for _, p := range t.ReturnParams {
 									fmt.Fprintf(os.Stderr, "=> %#v\n", p.Value)
 								}
-								delete(gFnEntrySeen, t.GoroutineID)
 							} else {
-								gFnEntrySeen[t.GoroutineID] = struct{}{}
 								fmt.Fprintf(os.Stderr, "> (%d) %s(%s)\n", t.GoroutineID, t.FunctionName, params.String())
 							}
 						}
@@ -713,7 +729,11 @@ func traceCmd(cmd *cobra.Command, args []string) {
 				}
 			}()
 		}
-		cmds.Call("continue", t)
+		err = cmds.Call("continue", t)
+		if err != nil {
+			fmt.Fprintln(os.Stderr, err)
+			return 1
+		}
 		return 0
 	}()
 	os.Exit(status)
