@@ -210,7 +210,11 @@ func getDlvBin(t *testing.T) (string, string) {
 	// we can ensure we don't get build errors
 	// depending on the test ordering.
 	os.Setenv("CGO_LDFLAGS", ldFlags)
-	return getDlvBinInternal(t)
+	var tags string
+	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
+		tags = "-tags=exp.winarm64"
+	}
+	return getDlvBinInternal(t, tags)
 }
 
 func getDlvBinEBPF(t *testing.T) (string, string) {
@@ -424,6 +428,10 @@ func TestGeneratedDoc(t *testing.T) {
 	if strings.ToLower(os.Getenv("TRAVIS")) == "true" && runtime.GOOS == "windows" {
 		t.Skip("skipping test on Windows in CI")
 	}
+	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
+		//TODO(qmuntal): investigate further when the Windows ARM64 backend is more stable.
+		t.Skip("skipping test on Windows in CI")
+	}
 	// Checks gen-cli-docs.go
 	var generatedBuf bytes.Buffer
 	commands := terminal.DebugCommands(nil)
@@ -457,12 +465,13 @@ func TestGeneratedDoc(t *testing.T) {
 		return out
 	}
 
+	checkAutogenDoc(t, "Documentation/backend_test_health.md", "go run _scripts/gen-backend_test_health.go", runScript("_scripts/gen-backend_test_health.go", "-"))
 	checkAutogenDoc(t, "pkg/terminal/starbind/starlark_mapping.go", "'go generate' inside pkg/terminal/starbind", runScript("_scripts/gen-starlark-bindings.go", "go", "-"))
 	checkAutogenDoc(t, "Documentation/cli/starlark.md", "'go generate' inside pkg/terminal/starbind", runScript("_scripts/gen-starlark-bindings.go", "doc/dummy", "Documentation/cli/starlark.md"))
-	checkAutogenDoc(t, "Documentation/backend_test_health.md", "go run _scripts/gen-backend_test_health.go", runScript("_scripts/gen-backend_test_health.go", "-"))
-	checkAutogenDoc(t, "_scripts/rtype-out.txt", "go run _scripts/rtype.go report _scripts/rtype-out.txt", runScript("_scripts/rtype.go", "report"))
-
-	runScript("_scripts/rtype.go", "check")
+	if goversion.VersionAfterOrEqual(runtime.Version(), 1, 18) {
+		checkAutogenDoc(t, "_scripts/rtype-out.txt", "go run _scripts/rtype.go report _scripts/rtype-out.txt", runScript("_scripts/rtype.go", "report"))
+		runScript("_scripts/rtype.go", "check")
+	}
 }
 
 func TestExitInInit(t *testing.T) {
@@ -700,8 +709,15 @@ func TestDAPCmd(t *testing.T) {
 	client.DisconnectRequest()
 	client.ExpectDisconnectResponse(t)
 	client.ExpectTerminatedEvent(t)
-	if _, err := client.ReadMessage(); err != io.EOF {
-		t.Errorf("got %q, want \"EOF\"\n", err)
+	_, err = client.ReadMessage()
+	if runtime.GOOS == "windows" {
+		if err == nil {
+			t.Errorf("got %q, want non-nil\n", err)
+		}
+	} else {
+		if err != io.EOF {
+			t.Errorf("got %q, want \"EOF\"\n", err)
+		}
 	}
 	client.Close()
 	cmd.Wait()
@@ -1020,6 +1036,11 @@ func TestTrace(t *testing.T) {
 }
 
 func TestTraceMultipleGoroutines(t *testing.T) {
+	if runtime.GOOS == "freebsd" {
+		//TODO(aarzilli): investigate further when the FreeBSD backend is more stable.
+		t.Skip("temporarily disabled due to issues with FreeBSD in Delve and Go")
+	}
+
 	dlvbin, tmpdir := getDlvBin(t)
 	defer os.RemoveAll(tmpdir)
 
@@ -1284,6 +1305,10 @@ func TestStaticcheck(t *testing.T) {
 	if err != nil {
 		t.Skip("staticcheck not installed")
 	}
+	if goversion.VersionAfterOrEqual(runtime.Version(), 1, 20) {
+		//TODO(aarzilli): remove this when there is a version of staticcheck that can support go1.20
+		t.Skip("staticcheck does not support go1.20 currently")
+	}
 	// default checks minus SA1019 which complains about deprecated identifiers, which change between versions of Go.
 	args := []string{"-tests=false", "-checks=all,-SA1019,-ST1000,-ST1003,-ST1016,-S1021,-ST1023", "github.com/go-delve/delve/..."}
 	// * SA1019 is disabled because new deprecations get added on every version
