@@ -270,6 +270,10 @@ func packageMatch(specPkg, symPkg string, packageMap map[string][]string) bool {
 // Find will search all functions in the target program and filter them via the
 // regex location spec. Only functions matching the regex will be returned.
 func (loc *RegexLocationSpec) Find(t *proc.Target, _ []string, scope *proc.EvalScope, locStr string, includeNonExecutableLines bool, _ [][2]string) ([]api.Location, error) {
+	if scope == nil {
+		//TODO(aarzilli): this needs only the list of function we should make it work
+		return nil, fmt.Errorf("could not determine location (scope is nil)")
+	}
 	funcs := scope.BinInfo.Functions
 	matches, err := regexFilterFuncs(loc.FuncRegex, funcs)
 	if err != nil {
@@ -370,7 +374,7 @@ func (ale AmbiguousLocationError) Error() string {
 func (loc *NormalLocationSpec) Find(t *proc.Target, processArgs []string, scope *proc.EvalScope, locStr string, includeNonExecutableLines bool, substitutePathRules [][2]string) ([]api.Location, error) {
 	limit := maxFindLocationCandidates
 	var candidateFiles []string
-	for _, sourceFile := range scope.BinInfo.Sources {
+	for _, sourceFile := range t.BinInfo().Sources {
 		substFile := sourceFile
 		if len(substitutePathRules) > 0 {
 			substFile = SubstitutePath(sourceFile, substitutePathRules)
@@ -387,10 +391,13 @@ func (loc *NormalLocationSpec) Find(t *proc.Target, processArgs []string, scope
 
 	var candidateFuncs []string
 	if loc.FuncBase != nil && limit > 0 {
-		candidateFuncs = loc.findFuncCandidates(scope, limit)
+		candidateFuncs = loc.findFuncCandidates(t.BinInfo(), limit)
 	}
 
 	if matching := len(candidateFiles) + len(candidateFuncs); matching == 0 {
+		if scope == nil {
+			return nil, fmt.Errorf("location \"%s\" not found", locStr)
+		}
 		// if no result was found this locations string could be an
 		// expression that the user forgot to prefix with '*', try treating it as
 		// such.
@@ -428,14 +435,14 @@ func (loc *NormalLocationSpec) Find(t *proc.Target, processArgs []string, scope
 	return []api.Location{addressesToLocation(addrs)}, nil
 }
 
-func (loc *NormalLocationSpec) findFuncCandidates(scope *proc.EvalScope, limit int) []string {
+func (loc *NormalLocationSpec) findFuncCandidates(bi *proc.BinaryInfo, limit int) []string {
 	candidateFuncs := map[string]struct{}{}
 	// See if it matches generic functions first
-	for fname := range scope.BinInfo.LookupGenericFunc() {
+	for fname := range bi.LookupGenericFunc() {
 		if len(candidateFuncs) >= limit {
 			break
 		}
-		if !loc.FuncBase.Match(&proc.Function{Name: fname}, scope.BinInfo.PackageMap) {
+		if !loc.FuncBase.Match(&proc.Function{Name: fname}, bi.PackageMap) {
 			continue
 		}
 		if loc.Base == fname {
@@ -443,11 +450,11 @@ func (loc *NormalLocationSpec) findFuncCandidates(scope *proc.EvalScope, limit i
 		}
 		candidateFuncs[fname] = struct{}{}
 	}
-	for _, f := range scope.BinInfo.LookupFunc {
+	for _, f := range bi.LookupFunc {
 		if len(candidateFuncs) >= limit {
 			break
 		}
-		if !loc.FuncBase.Match(f, scope.BinInfo.PackageMap) {
+		if !loc.FuncBase.Match(f, bi.PackageMap) {
 			continue
 		}
 		if loc.Base == f.Name {
@@ -492,13 +499,25 @@ func SubstitutePath(path string, rules [][2]string) string {
 		from := crossPlatformPath(r[0])
 		to := r[1]
 
-		if !strings.HasSuffix(from, separator) {
+		// If we have an exact match, use it directly.
+		if path == from {
+			return to
+		}
+
+		// Otherwise check if it's a directory prefix.
+		if from != "" && !strings.HasSuffix(from, separator) {
 			from = from + separator
 		}
-		if !strings.HasSuffix(to, separator) {
+		if to != "" && !strings.HasSuffix(to, separator) {
 			to = to + separator
 		}
-		if strings.HasPrefix(path, from) {
+
+		// Expand relative paths with the specified prefix
+		if from == "" && !filepath.IsAbs(path) {
+			return strings.Replace(path, from, to, 1)
+		}
+
+		if from != "" && strings.HasPrefix(path, from) {
 			return strings.Replace(path, from, to, 1)
 		}
 	}
