@@ -12,7 +12,9 @@ import (
 	"fmt"
 	"go/ast"
 	"go/token"
+	"hash/crc32"
 	"io"
+	"io/ioutil"
 	"os"
 	"path/filepath"
 	"sort"
@@ -92,12 +94,8 @@ type BinaryInfo struct {
 	types       map[string]dwarfRef
 	packageVars []packageVar // packageVars is a list of all global/package variables in debug_info, sorted by address
 
-	gStructOffset uint64
-
-	// nameOfRuntimeType maps an address of a runtime._type struct to its
-	// decoded name. Used with versions of Go <= 1.10 to figure out the DIE of
-	// the concrete type of interfaces.
-	nameOfRuntimeType map[uint64]nameOfRuntimeTypeEntry
+	gStructOffset      uint64
+	gStructOffsetIsPtr bool
 
 	// consts[off] lists all the constants with the type defined at offset off.
 	consts constantsMap
@@ -135,6 +133,7 @@ var (
 
 	supportedWindowsArch = map[_PEMachine]bool{
 		_IMAGE_FILE_MACHINE_AMD64: true,
+		_IMAGE_FILE_MACHINE_ARM64: true,
 	}
 
 	supportedDarwinArch = map[macho.Cpu]bool{
@@ -191,7 +190,7 @@ func FindFileLocation(p Process, filename string, lineno int) ([]uint64, error)
 		return nil, &ErrCouldNotFindLine{fileFound, filename, lineno}
 	}
 
-	// 2. assign all occurences of filename:lineno to their containing function
+	// 2. assign all occurrences of filename:lineno to their containing function
 
 	pcByFunc := map[*Function][]line.PCStmt{}
 	sort.Slice(pcs, func(i, j int) bool { return pcs[i].PC < pcs[j].PC })
@@ -394,6 +393,27 @@ func FirstPCAfterPrologue(p Process, fn *Function, sameline bool) (uint64, error
 	return pc, nil
 }
 
+func findRetPC(t *Target, name string) ([]uint64, error) {
+	fn := t.BinInfo().LookupFunc[name]
+	if fn == nil {
+		return nil, fmt.Errorf("could not find %s", name)
+	}
+	text, err := Disassemble(t.Memory(), nil, t.Breakpoints(), t.BinInfo(), fn.Entry, fn.End)
+	if err != nil {
+		return nil, err
+	}
+	r := []uint64{}
+	for _, instr := range text {
+		if instr.IsRet() {
+			r = append(r, instr.Loc.PC)
+		}
+	}
+	if len(r) == 0 {
+		return nil, fmt.Errorf("could not find return instruction in %s", name)
+	}
+	return r, nil
+}
+
 // cpuArch is a stringer interface representing CPU architectures.
 type cpuArch interface {
 	String() string
@@ -636,7 +656,7 @@ type ElfDynamicSection struct {
 
 // NewBinaryInfo returns an initialized but unloaded BinaryInfo struct.
 func NewBinaryInfo(goos, goarch string) *BinaryInfo {
-	r := &BinaryInfo{GOOS: goos, nameOfRuntimeType: make(map[uint64]nameOfRuntimeTypeEntry), logger: logflags.DebuggerLogger()}
+	r := &BinaryInfo{GOOS: goos, logger: logflags.DebuggerLogger()}
 
 	// TODO: find better way to determine proc arch (perhaps use executable file info).
 	switch goarch {
@@ -679,8 +699,18 @@ func loadBinaryInfo(bi *BinaryInfo, image *Image, path string, entryPoint uint64
 
 // GStructOffset returns the offset of the G
 // struct in thread local storage.
-func (bi *BinaryInfo) GStructOffset() uint64 {
-	return bi.gStructOffset
+func (bi *BinaryInfo) GStructOffset(mem MemoryReadWriter) (uint64, error) {
+	offset := bi.gStructOffset
+	if bi.gStructOffsetIsPtr {
+		// The G struct offset from the TLS section is a pointer
+		// and the address must be dereferenced to find to actual G struct offset.
+		var err error
+		offset, err = readUintRaw(mem, offset, int64(bi.Arch.PtrSize()))
+		if err != nil {
+			return 0, err
+		}
+	}
+	return offset, nil
 }
 
 // LastModified returns the last modified time of the binary.
@@ -975,7 +1005,7 @@ func (bi *BinaryInfo) locationExpr(entry godwarf.Entry, attr dwarf.Attr, pc uint
 		return nil, nil, fmt.Errorf("no location attribute %s", attr)
 	}
 	if instr, ok := a.([]byte); ok {
-		return instr, &locationExpr{isBlock: true, instr: instr}, nil
+		return instr, &locationExpr{isBlock: true, instr: instr, regnumToName: bi.Arch.RegnumToString}, nil
 	}
 	off, ok := a.(int64)
 	if !ok {
@@ -985,7 +1015,7 @@ func (bi *BinaryInfo) locationExpr(entry godwarf.Entry, attr dwarf.Attr, pc uint
 	if instr == nil {
 		return nil, nil, fmt.Errorf("could not find loclist entry at %#x for address %#x", off, pc)
 	}
-	return instr, &locationExpr{pc: pc, off: off, instr: instr}, nil
+	return instr, &locationExpr{pc: pc, off: off, instr: instr, regnumToName: bi.Arch.RegnumToString}, nil
 }
 
 type locationExpr struct {
@@ -994,6 +1024,8 @@ type locationExpr struct {
 	off       int64
 	pc        uint64
 	instr     []byte
+
+	regnumToName func(uint64) string
 }
 
 func (le *locationExpr) String() string {
@@ -1004,10 +1036,10 @@ func (le *locationExpr) String() string {
 
 	if le.isBlock {
 		fmt.Fprintf(&descr, "[block] ")
-		op.PrettyPrint(&descr, le.instr)
+		op.PrettyPrint(&descr, le.instr, le.regnumToName)
 	} else {
 		fmt.Fprintf(&descr, "[%#x:%#x] ", le.off, le.pc)
-		op.PrettyPrint(&descr, le.instr)
+		op.PrettyPrint(&descr, le.instr, le.regnumToName)
 	}
 
 	if le.isEscaped {
@@ -1205,34 +1237,105 @@ func (bi *BinaryInfo) parseDebugFrameGeneral(image *Image, debugFrameBytes []byt
 // Alternatively, if the debug file cannot be found be the build-id, Delve
 // will look in directories specified by the debug-info-directories config value.
 func (bi *BinaryInfo) openSeparateDebugInfo(image *Image, exe *elf.File, debugInfoDirectories []string) (*os.File, *elf.File, error) {
-	var debugFilePath string
-	var err error
-	for _, dir := range debugInfoDirectories {
-		var potentialDebugFilePath string
-		if strings.Contains(dir, "build-id") {
-			potentialDebugFilePath = fmt.Sprintf("%s/%s/%s.debug", dir, bi.BuildID[:2], bi.BuildID[2:])
-		} else if strings.HasPrefix(image.Path, "/proc") {
-			path, err := filepath.EvalSymlinks(image.Path)
-			if err == nil {
-				potentialDebugFilePath = fmt.Sprintf("%s/%s.debug", dir, filepath.Base(path))
-			}
-		} else {
-			potentialDebugFilePath = fmt.Sprintf("%s/%s.debug", dir, filepath.Base(image.Path))
+	exePath := image.Path
+	exeName := filepath.Base(image.Path)
+	if strings.HasPrefix(image.Path, "/proc") {
+		var err error
+		exePath, err = filepath.EvalSymlinks(image.Path)
+		if err == nil {
+			exeName = filepath.Base(exePath)
 		}
+	}
+
+	var debugFilePath string
+
+	check := func(potentialDebugFilePath string) bool {
 		_, err := os.Stat(potentialDebugFilePath)
 		if err == nil {
 			debugFilePath = potentialDebugFilePath
-			break
+			return true
+		}
+		return false
+	}
+
+	find := func(f func(string) bool, suffix string) {
+		for _, dir := range debugInfoDirectories {
+			if f != nil && !f(dir) {
+				continue
+			}
+			if check(fmt.Sprintf("%s/%s", dir, suffix)) {
+				break
+			}
 		}
 	}
+
+	if debugFilePath == "" && len(bi.BuildID) > 2 {
+		// Build ID method: look for a file named .build-id/nn/nnnnnnnn.debug in
+		// every debug info directory.
+		find(nil, fmt.Sprintf(".build-id/%s/%s.debug", bi.BuildID[:2], bi.BuildID[2:]))
+	}
+
+	if debugFilePath == "" {
+		// Debug link: method if the executable contains a .gnu_debuglink section
+		// it will look for the file named in the same directory of the
+		// executable, then in a subdirectory named .debug and finally in each
+		// debug info directory in a subdirectory with the same path as the
+		// directory of the executable
+		debugLink, crc := bi.getDebugLink(exe)
+
+		if debugLink != "" {
+			check(filepath.Join(filepath.Dir(exePath), debugLink))
+			if debugFilePath == "" {
+				check(filepath.Join(filepath.Dir(exePath), ".debug", debugLink))
+			}
+			if debugFilePath == "" {
+				suffix := filepath.Join(filepath.Dir(exePath)[1:], debugLink)
+				find(nil, suffix)
+			}
+			if debugFilePath == "" {
+				bi.logger.Warnf("gnu_debuglink link %q not found in any debug info directory", debugLink)
+			}
+		}
+
+		if debugFilePath != "" {
+			// CRC check
+			buf, err := ioutil.ReadFile(debugFilePath)
+			if err == nil {
+				computedCRC := crc32.ChecksumIEEE(buf)
+				if crc != computedCRC {
+					bi.logger.Errorf("gnu_debuglink CRC check failed for %s (want %x got %x)", debugFilePath, crc, computedCRC)
+					debugFilePath = ""
+				}
+
+			}
+		}
+	}
+
+	if debugFilePath == "" && len(bi.BuildID) > 2 {
+		// Previous verrsions of delve looked for the build id in every debug info
+		// directory that contained the build-id substring. This behavior deviates
+		// from the ones specified by GDB but we keep it for backwards compatibility.
+		find(func(dir string) bool { return strings.Contains(dir, "build-id") }, fmt.Sprintf("%s/%s.debug", bi.BuildID[:2], bi.BuildID[2:]))
+	}
+
+	if debugFilePath == "" {
+		// Previous versions of delve looked for the executable filename (with
+		// .debug extension) in every debug info directory.  This behavior also
+		// deviates from the ones specified by GDB, but we keep it for backwards
+		// compatibility.
+		find(func(dir string) bool { return !strings.Contains(dir, "build-id") }, fmt.Sprintf("%s.debug", exeName))
+	}
+
 	// We cannot find the debug information locally on the system. Try and see if we're on a system that
 	// has debuginfod so that we can use that in order to find any relevant debug information.
 	if debugFilePath == "" {
+		var err error
 		debugFilePath, err = debuginfod.GetDebuginfo(bi.BuildID)
 		if err != nil {
 			return nil, nil, ErrNoDebugInfoFound
 		}
 	}
+
 	sepFile, err := os.OpenFile(debugFilePath, 0, os.ModePerm)
 	if err != nil {
 		return nil, nil, errors.New("can't open separate debug file: " + err.Error())
@@ -1337,7 +1440,7 @@ func loadBinaryInfoElf(bi *BinaryInfo, image *Image, path string, addr uint64, w
 	return nil
 }
 
-//  _STT_FUNC is a code object, see /usr/include/elf.h for a full definition.
+// _STT_FUNC is a code object, see /usr/include/elf.h for a full definition.
 const _STT_FUNC = 2
 
 func (bi *BinaryInfo) loadSymbolName(image *Image, file *elf.File, wg *sync.WaitGroup) {
@@ -1357,36 +1460,57 @@ func (bi *BinaryInfo) loadSymbolName(image *Image, file *elf.File, wg *sync.Wait
 func (bi *BinaryInfo) loadBuildID(image *Image, file *elf.File) {
 	buildid := file.Section(".note.gnu.build-id")
 	if buildid == nil {
-		bi.logger.Error("can't find build-id note on binary")
 		return
 	}
 
 	br := buildid.Open()
 	bh := new(buildIDHeader)
 	if err := binary.Read(br, binary.LittleEndian, bh); err != nil {
-		bi.logger.Errorf("can't read build-id header: %v", err)
+		bi.logger.Warnf("can't read build-id header: %v", err)
 		return
 	}
 
 	name := make([]byte, bh.Namesz)
 	if err := binary.Read(br, binary.LittleEndian, name); err != nil {
-		bi.logger.Errorf("can't read build-id name: %v", err)
+		bi.logger.Warnf("can't read build-id name: %v", err)
 		return
 	}
 
 	if strings.TrimSpace(string(name)) != "GNU\x00" {
-		bi.logger.Error("invalid build-id signature")
+		bi.logger.Warn("invalid build-id signature")
 		return
 	}
 
 	descBinary := make([]byte, bh.Descsz)
 	if err := binary.Read(br, binary.LittleEndian, descBinary); err != nil {
-		bi.logger.Errorf("can't read build-id desc: %v", err)
+		bi.logger.Warnf("can't read build-id desc: %v", err)
 		return
 	}
 	bi.BuildID = hex.EncodeToString(descBinary)
 }
 
+func (bi *BinaryInfo) getDebugLink(exe *elf.File) (debugLink string, crc uint32) {
+	gnuDebugLink := exe.Section(".gnu_debuglink")
+	if gnuDebugLink == nil {
+		return
+	}
+
+	br := gnuDebugLink.Open()
+	buf, err := ioutil.ReadAll(br)
+	if err != nil {
+		bi.logger.Warnf("can't read .gnu_debuglink: %v", err)
+		return
+	}
+	zero := bytes.Index(buf, []byte{0})
+	if zero <= 0 || len(buf[zero+1:]) < 4 {
+		bi.logger.Warnf("wrong .gnu_debuglink format: %q", buf)
+		return
+	}
+	debugLink = string(buf[:zero])
+	crc = binary.LittleEndian.Uint32(buf[len(buf)-4:])
+	return
+}
+
 func (bi *BinaryInfo) parseDebugFrameElf(image *Image, dwarfFile, exeFile *elf.File, debugInfoBytes []byte, wg *sync.WaitGroup) {
 	defer wg.Done()
 
@@ -1396,7 +1520,12 @@ func (bi *BinaryInfo) parseDebugFrameElf(image *Image, dwarfFile, exeFile *elf.F
 	var ehFrameAddr uint64
 	if ehFrameSection != nil {
 		ehFrameAddr = ehFrameSection.Addr
-		ehFrameData, _ = ehFrameSection.Data()
+		// Workaround for go.dev/cl/429601
+		if ehFrameSection.Type == elf.SHT_NOBITS {
+			ehFrameData = make([]byte, ehFrameSection.Size)
+		} else {
+			ehFrameData, _ = ehFrameSection.Data()
+		}
 	}
 
 	bi.parseDebugFrameGeneral(image, debugFrameData, ".debug_frame", debugFrameErr, ehFrameData, ehFrameAddr, ".eh_frame", frame.DwarfEndian(debugInfoBytes))
@@ -1496,8 +1625,6 @@ func loadBinaryInfoPE(bi *BinaryInfo, image *Image, path string, entryPoint uint
 	if err != nil {
 		return err
 	}
-
-	//TODO(aarzilli): actually test this when Go supports PIE buildmode on Windows.
 	opth := peFile.OptionalHeader.(*pe.OptionalHeader64)
 	if entryPoint != 0 {
 		image.StaticBase = entryPoint - opth.ImageBase
@@ -1524,16 +1651,55 @@ func loadBinaryInfoPE(bi *BinaryInfo, image *Image, path string, entryPoint uint
 
 	wg.Add(2)
 	go bi.parseDebugFramePE(image, peFile, debugInfoBytes, wg)
-	go bi.loadDebugInfoMaps(image, debugInfoBytes, debugLineBytes, wg, nil)
-
-	// Use ArbitraryUserPointer (0x28) as pointer to pointer
-	// to G struct per:
-	// https://golang.org/src/runtime/cgo/gcc_windows_amd64.c
-
-	bi.gStructOffset = 0x28
+	go bi.loadDebugInfoMaps(image, debugInfoBytes, debugLineBytes, wg, func() {
+		// setGStructOffsetPE requires the image compile units to be loaded,
+		// so it can't be called concurrently with loadDebugInfoMaps.
+		if image.index == 0 {
+			// determine g struct offset only when loading the executable file.
+			bi.setGStructOffsetPE(entryPoint, peFile)
+		}
+	})
 	return nil
 }
 
+func (bi *BinaryInfo) setGStructOffsetPE(entryPoint uint64, peFile *pe.File) {
+	readtls_g := func() uint64 {
+		for _, s := range peFile.Symbols {
+			if s.Name == "runtime.tls_g" {
+				i := int(s.SectionNumber) - 1
+				if 0 <= i && i < len(peFile.Sections) {
+					sect := peFile.Sections[i]
+					if s.Value < sect.VirtualSize {
+						return entryPoint + uint64(sect.VirtualAddress) + uint64(s.Value)
+					}
+				}
+				break
+			}
+		}
+		return 0
+	}
+	switch _PEMachine(peFile.Machine) {
+	case _IMAGE_FILE_MACHINE_AMD64:
+		producer := bi.Producer()
+		if producer != "" && goversion.ProducerAfterOrEqual(producer, 1, 20) {
+			// Use runtime.tls_g as pointer to offset from GS to G struct:
+			// https://golang.org/src/runtime/sys_windows_amd64.s
+			bi.gStructOffset = readtls_g()
+			bi.gStructOffsetIsPtr = true
+		} else {
+			// Use ArbitraryUserPointer (0x28) as pointer to pointer
+			// to G struct per:
+			// https://golang.org/src/runtime/cgo/gcc_windows_amd64.c
+			bi.gStructOffset = 0x28
+		}
+	case _IMAGE_FILE_MACHINE_ARM64:
+		// Use runtime.tls_g as pointer to offset from R18 to G struct:
+		// https://golang.org/src/runtime/sys_windows_arm64.s
+		bi.gStructOffset = readtls_g()
+		bi.gStructOffsetIsPtr = true
+	}
+}
+
 func openExecutablePathPE(path string) (*pe.File, io.Closer, error) {
 	f, err := os.OpenFile(path, 0, os.ModePerm)
 	if err != nil {
@@ -1633,16 +1799,17 @@ func (bi *BinaryInfo) parseDebugFrameMacho(image *Image, exe *macho.File, debugI
 	bi.parseDebugFrameGeneral(image, debugFrameBytes, "__debug_frame", debugFrameErr, ehFrameBytes, ehFrameAddr, "__eh_frame", frame.DwarfEndian(debugInfoBytes))
 }
 
-// macOSDebugFrameBugWorkaround applies a workaround for:
-//  https://github.com/golang/go/issues/25841
+// macOSDebugFrameBugWorkaround applies a workaround for [golang/go#25841]
+//
 // It finds the Go function with the lowest entry point and the first
 // debug_frame FDE, calculates the difference between the start of the
 // function and the start of the FDE and sums it to all debug_frame FDEs.
 // A number of additional checks are performed to make sure we don't ruin
 // executables unaffected by this bug.
+//
+// [golang/go#25841]: https://github.com/golang/go/issues/25841
 func (bi *BinaryInfo) macOSDebugFrameBugWorkaround() {
-	//TODO: log extensively because of bugs in the field
-	if bi.GOOS != "darwin" || bi.Arch.Name != "arm64" {
+	if bi.GOOS != "darwin" {
 		return
 	}
 	if len(bi.Images) > 1 {
@@ -1655,9 +1822,28 @@ func (bi *BinaryInfo) macOSDebugFrameBugWorkaround() {
 	if !ok {
 		return
 	}
-	if exe.Flags&macho.FlagPIE == 0 {
-		bi.logger.Infof("debug_frame workaround not needed: not a PIE (%#x)", exe.Flags)
-		return
+	if bi.Arch.Name == "arm64" {
+		if exe.Flags&macho.FlagPIE == 0 {
+			bi.logger.Infof("debug_frame workaround not needed: not a PIE (%#x)", exe.Flags)
+			return
+		}
+	} else {
+		prod := goversion.ParseProducer(bi.Producer())
+		if !prod.AfterOrEqual(goversion.GoVersion{Major: 1, Minor: 19, Rev: 3}) && !prod.IsDevel() {
+			bi.logger.Infof("debug_frame workaround not needed (version %q on %s)", bi.Producer(), bi.Arch.Name)
+			return
+		}
+		found := false
+		for i := range bi.frameEntries {
+			if bi.frameEntries[i].CIE.CIE_id == ^uint32(0) && bi.frameEntries[i].Begin() < 0x4000000 {
+				found = true
+				break
+			}
+		}
+		if !found {
+			bi.logger.Infof("debug_frame workaround not needed (all FDEs above 0x4000000)")
+			return
+		}
 	}
 
 	// Find first Go function (first = lowest entry point)
@@ -1713,6 +1899,8 @@ func (bi *BinaryInfo) macOSDebugFrameBugWorkaround() {
 
 // Do not call this function directly it isn't able to deal correctly with package paths
 func (bi *BinaryInfo) findType(name string) (godwarf.Type, error) {
+	name = strings.Replace(name, "interface{", "interface {", -1)
+	name = strings.Replace(name, "struct{", "struct {", -1)
 	ref, found := bi.types[name]
 	if !found {
 		return nil, reader.ErrTypeNotFound
@@ -1967,9 +2155,9 @@ func (bi *BinaryInfo) loadDebugInfoMaps(image *Image, debugInfoBytes, debugLineB
 
 // LookupGenericFunc returns a map that allows searching for instantiations of generic function by specificying a function name without type parameters.
 // For example the key "pkg.(*Receiver).Amethod" will find all instantiations of Amethod:
-//  - pkg.(*Receiver[.shape.int]).Amethod"
-//  - pkg.(*Receiver[.shape.*uint8]).Amethod"
-//  - etc.
+//   - pkg.(*Receiver[.shape.int]).Amethod
+//   - pkg.(*Receiver[.shape.*uint8]).Amethod
+//   - etc.
 func (bi *BinaryInfo) LookupGenericFunc() map[string][]*Function {
 	if bi.lookupGenericFunc == nil {
 		bi.lookupGenericFunc = make(map[string][]*Function)
