@@ -54,7 +54,7 @@ const logCommandOutput = false
 
 func (ft *FakeTerminal) Exec(cmdstr string) (outstr string, err error) {
 	var buf bytes.Buffer
-	ft.Term.stdout.w = &buf
+	ft.Term.stdout.pw.w = &buf
 	ft.Term.starlarkEnv.Redirect(ft.Term.stdout)
 	err = ft.cmds.Call(cmdstr, ft.Term)
 	outstr = buf.String()
@@ -67,7 +67,7 @@ func (ft *FakeTerminal) Exec(cmdstr string) (outstr string, err error) {
 
 func (ft *FakeTerminal) ExecStarlark(starlarkProgram string) (outstr string, err error) {
 	var buf bytes.Buffer
-	ft.Term.stdout.w = &buf
+	ft.Term.stdout.pw.w = &buf
 	ft.Term.starlarkEnv.Redirect(ft.Term.stdout)
 	_, err = ft.Term.starlarkEnv.Execute("<stdin>", starlarkProgram, "main", nil)
 	outstr = buf.String()
@@ -155,7 +155,7 @@ func withTestTerminalBuildFlags(name string, t testing.TB, buildFlags test.Build
 func TestCommandDefault(t *testing.T) {
 	var (
 		cmds = Commands{}
-		cmd  = cmds.Find("non-existant-command", noPrefix).cmdFn
+		cmd  = cmds.Find("non-existent-command", noPrefix).cmdFn
 	)
 
 	err := cmd(nil, callContext{}, "")
@@ -1158,6 +1158,46 @@ func TestHitCondBreakpoint(t *testing.T) {
 			t.Fatalf("wrong value of i")
 		}
 	})
+
+	withTestTerminal("condperghitcount", t, func(term *FakeTerminal) {
+		term.MustExec("break bp1 main.main:8")
+		term.MustExec("condition -per-g-hitcount bp1 == 2")
+		listIsAt(t, term, "continue", 16, -1, -1)
+		// first g hit
+		out := term.MustExec("print j")
+		t.Logf("%q", out)
+		if !strings.Contains(out, "2\n") {
+			t.Fatalf("wrong value of j")
+		}
+		term.MustExec("toggle bp1")
+		listIsAt(t, term, "continue", 16, -1, -1)
+		// second g hit
+		out = term.MustExec("print j")
+		t.Logf("%q", out)
+		if !strings.Contains(out, "2\n") {
+			t.Fatalf("wrong value of j")
+		}
+	})
+}
+
+func TestClearCondBreakpoint(t *testing.T) {
+	withTestTerminal("break", t, func(term *FakeTerminal) {
+		term.MustExec("break main.main:4")
+		term.MustExec("condition 1 i%3==2")
+		listIsAt(t, term, "continue", 7, -1, -1)
+		out := term.MustExec("print i")
+		t.Logf("%q", out)
+		if !strings.Contains(out, "2\n") {
+			t.Fatalf("wrong value of i")
+		}
+		term.MustExec("condition -clear 1")
+		listIsAt(t, term, "continue", 7, -1, -1)
+		out = term.MustExec("print i")
+		t.Logf("%q", out)
+		if !strings.Contains(out, "3\n") {
+			t.Fatalf("wrong value of i")
+		}
+	})
 }
 
 func TestBreakpointEditing(t *testing.T) {
@@ -1291,3 +1331,28 @@ func TestTranscript(t *testing.T) {
 		os.Remove(name)
 	})
 }
+
+func TestDisassPosCmd(t *testing.T) {
+	withTestTerminal("testvariables2", t, func(term *FakeTerminal) {
+		term.MustExec("continue")
+		out := term.MustExec("step-instruction")
+		t.Logf("%q\n", out)
+		if !strings.Contains(out, "call $runtime.Breakpoint") && !strings.Contains(out, "CALL runtime.Breakpoint(SB)") {
+			t.Errorf("output doesn't look like disassembly")
+		}
+	})
+}
+
+func TestCreateBreakpointByLocExpr(t *testing.T) {
+	withTestTerminal("math", t, func(term *FakeTerminal) {
+		out := term.MustExec("break main.main")
+		position1 := strings.Split(out, " set at ")[1]
+		term.MustExec("continue")
+		term.MustExec("clear 1")
+		out = term.MustExec("break +0")
+		position2 := strings.Split(out, " set at ")[1]
+		if position1 != position2 {
+			t.Fatalf("mismatched positions %q and %q\n", position1, position2)
+		}
+	})
+}
