@@ -55,12 +55,12 @@ func TestMain(m *testing.M) {
 }
 
 func withTestClient2(name string, t *testing.T, fn func(c service.Client)) {
-	withTestClient2Extended(name, t, 0, [3]string{}, func(c service.Client, fixture protest.Fixture) {
+	withTestClient2Extended(name, t, 0, [3]string{}, nil, func(c service.Client, fixture protest.Fixture) {
 		fn(c)
 	})
 }
 
-func startServer(name string, buildFlags protest.BuildFlags, t *testing.T, redirects [3]string) (clientConn net.Conn, fixture protest.Fixture) {
+func startServer(name string, buildFlags protest.BuildFlags, t *testing.T, redirects [3]string, args []string) (clientConn net.Conn, fixture protest.Fixture) {
 	if testBackend == "rr" {
 		protest.MustHaveRecordingAllowed(t)
 	}
@@ -77,7 +77,7 @@ func startServer(name string, buildFlags protest.BuildFlags, t *testing.T, redir
 	}
 	server := rpccommon.NewServer(&service.Config{
 		Listener:    listener,
-		ProcessArgs: []string{fixture.Path},
+		ProcessArgs: append([]string{fixture.Path}, args...),
 		Debugger: debugger.Config{
 			Backend:        testBackend,
 			CheckGoVersion: true,
@@ -93,8 +93,8 @@ func startServer(name string, buildFlags protest.BuildFlags, t *testing.T, redir
 	return clientConn, fixture
 }
 
-func withTestClient2Extended(name string, t *testing.T, buildFlags protest.BuildFlags, redirects [3]string, fn func(c service.Client, fixture protest.Fixture)) {
-	clientConn, fixture := startServer(name, buildFlags, t, redirects)
+func withTestClient2Extended(name string, t *testing.T, buildFlags protest.BuildFlags, redirects [3]string, args []string, fn func(c service.Client, fixture protest.Fixture)) {
+	clientConn, fixture := startServer(name, buildFlags, t, redirects, args)
 	client := rpc2.NewClientFromConn(clientConn)
 	defer func() {
 		client.Detach(true)
@@ -230,7 +230,7 @@ func TestRestart_rebuild(t *testing.T) {
 	// In the original fixture file the env var tested for is SOMEVAR.
 	os.Setenv("SOMEVAR", "bah")
 
-	withTestClient2Extended("testenv", t, 0, [3]string{}, func(c service.Client, f protest.Fixture) {
+	withTestClient2Extended("testenv", t, 0, [3]string{}, nil, func(c service.Client, f protest.Fixture) {
 		<-c.Continue()
 
 		var1, err := c.EvalVariable(api.EvalScope{GoroutineID: -1}, "x", normalLoadConfig)
@@ -1019,7 +1019,7 @@ func TestClientServer_FindLocations(t *testing.T) {
 		findLocationHelper(t, c, "main.stacktraceme", false, 1, stacktracemeAddr)
 	})
 
-	withTestClient2Extended("locationsUpperCase", t, 0, [3]string{}, func(c service.Client, fixture protest.Fixture) {
+	withTestClient2Extended("locationsUpperCase", t, 0, [3]string{}, nil, func(c service.Client, fixture protest.Fixture) {
 		// Upper case
 		findLocationHelper(t, c, "locationsUpperCase.go:6", false, 1, 0)
 
@@ -1724,7 +1724,7 @@ func TestClientServer_FpRegisters(t *testing.T) {
 		{"XMM12", "…[ZMM12hh] 0x3ff66666666666663ff4cccccccccccd"},
 	}
 	protest.AllowRecording(t)
-	withTestClient2Extended("fputest/", t, 0, [3]string{}, func(c service.Client, fixture protest.Fixture) {
+	withTestClient2Extended("fputest/", t, 0, [3]string{}, nil, func(c service.Client, fixture protest.Fixture) {
 		_, err := c.CreateBreakpoint(&api.Breakpoint{File: filepath.Join(fixture.BuildDir, "fputest.go"), Line: 25})
 		assertNoError(err, t, "CreateBreakpoint")
 
@@ -2276,7 +2276,7 @@ func (c *brokenRPCClient) call(method string, args, reply interface{}) error {
 }
 
 func TestUnknownMethodCall(t *testing.T) {
-	clientConn, _ := startServer("continuetestprog", 0, t, [3]string{})
+	clientConn, _ := startServer("continuetestprog", 0, t, [3]string{}, nil)
 	client := &brokenRPCClient{jsonrpc.NewClient(clientConn)}
 	client.call("SetApiVersion", api.SetAPIVersionIn{APIVersion: 2}, &api.SetAPIVersionOut{})
 	defer client.Detach(true)
@@ -2420,7 +2420,7 @@ func TestClearLogicalBreakpoint(t *testing.T) {
 	// Clearing a logical breakpoint should clear all associated physical
 	// breakpoints.
 	// Issue #1955.
-	withTestClient2Extended("testinline", t, protest.EnableInlining, [3]string{}, func(c service.Client, fixture protest.Fixture) {
+	withTestClient2Extended("testinline", t, protest.EnableInlining, [3]string{}, nil, func(c service.Client, fixture protest.Fixture) {
 		bp, err := c.CreateBreakpoint(&api.Breakpoint{FunctionName: "main.inlineThis"})
 		assertNoError(err, t, "CreateBreakpoint()")
 		t.Logf("breakpoint set at %#v", bp.Addrs)
@@ -2446,7 +2446,7 @@ func TestRedirects(t *testing.T) {
 		outfile = "redirect-output.txt"
 	)
 	protest.AllowRecording(t)
-	withTestClient2Extended("redirect", t, 0, [3]string{infile, outfile, ""}, func(c service.Client, fixture protest.Fixture) {
+	withTestClient2Extended("redirect", t, 0, [3]string{infile, outfile, ""}, nil, func(c service.Client, fixture protest.Fixture) {
 		outpath := filepath.Join(fixture.BuildDir, outfile)
 		<-c.Continue()
 		buf, err := ioutil.ReadFile(outpath)
@@ -2767,3 +2767,214 @@ func TestRestartRewindAfterEnd(t *testing.T) {
 		}
 	})
 }
+
+func TestClientServer_SinglelineStringFormattedWithBigInts(t *testing.T) {
+	// Check that variables that represent large numbers are represented correctly when using a formatting string
+
+	if runtime.GOARCH != "amd64" {
+		t.Skip("N/A")
+	}
+	withTestClient2Extended("xmm0print/", t, 0, [3]string{}, nil, func(c service.Client, fixture protest.Fixture) {
+		_, err := c.CreateBreakpoint(&api.Breakpoint{FunctionName: "main.VPSLLQ36", Line: 4})
+		assertNoError(err, t, "CreateBreakpoint")
+		state := <-c.Continue()
+		if state.CurrentThread.Line != 8 {
+			t.Fatalf("wrong location after continue %s:%d", state.CurrentThread.File, state.CurrentThread.Line)
+		}
+
+		constvar, err := c.EvalVariable(api.EvalScope{GoroutineID: -1}, "9331634762088972288", normalLoadConfig)
+		assertNoError(err, t, "ErrVariable(9331634762088972288)")
+		out := constvar.SinglelineStringFormatted("%X")
+		t.Logf("constant: %q\n", out)
+		if out != "8180A06000000000" {
+			t.Errorf("expected \"8180A06000000000\" got %q when printing constant", out)
+		}
+
+		xmm0var, err := c.EvalVariable(api.EvalScope{GoroutineID: -1}, "XMM0.uint64", normalLoadConfig)
+		assertNoError(err, t, "EvalVariable(XMM0.uint64)")
+
+		expected := []string{
+			"9331634762088972288", "8180A06000000000",
+			"9331634762088972288", "8180A06000000000",
+			"9259436018245828608", "8080200000000000",
+			"9259436018245828608", "8080200000000000",
+		}
+
+		for i := range xmm0var.Children {
+			child := &xmm0var.Children[i]
+			if child.Kind != reflect.Uint64 {
+				t.Errorf("wrong kind for variable %s\n", child.Kind)
+			}
+			out1 := child.SinglelineString()
+			out2 := child.SinglelineStringFormatted("%X")
+			t.Logf("%q %q\n", out1, out2)
+			if out1 != expected[i*2] {
+				t.Errorf("for child %d expected %s got %s (decimal)", i, expected[i*2], out1)
+			}
+			if out2 != expected[i*2+1] {
+				t.Errorf("for child %d expected %s got %s (hexadecimal)", i, expected[i*2+1], out2)
+			}
+		}
+	})
+}
+
+func TestNonGoDebug(t *testing.T) {
+	// Test that we can at least set breakpoints while debugging a non-go executable.
+	if runtime.GOOS != "linux" {
+		t.Skip()
+	}
+	dir := protest.FindFixturesDir()
+	path := protest.TempFile("testc")
+	cmd := exec.Command("cc", "-g", "-o", path, filepath.Join(dir, "test.c"))
+	if out, err := cmd.CombinedOutput(); err != nil {
+		t.Fatalf("Error compiling %s: %s\n%s", path, err, out)
+	}
+
+	listener, clientConn := service.ListenerPipe()
+	defer listener.Close()
+
+	server := rpccommon.NewServer(&service.Config{
+		Listener:    listener,
+		ProcessArgs: []string{path},
+		Debugger: debugger.Config{
+			Backend:     testBackend,
+			ExecuteKind: debugger.ExecutingExistingFile,
+		},
+	})
+
+	if err := server.Run(); err != nil {
+		t.Fatal(err)
+	}
+
+	client := rpc2.NewClientFromConn(clientConn)
+	defer func() {
+		client.Detach(true)
+	}()
+
+	_, err := client.CreateBreakpoint(&api.Breakpoint{FunctionName: "C.main", Line: -1})
+	if err != nil {
+		t.Fatal(err)
+	}
+}
+
+func TestRestart_PreserveFunctionBreakpoint(t *testing.T) {
+	// Tests that function breakpoint get restored correctly, after a rebuild,
+	// even if the function changed position in the source file.
+
+	dir := protest.FindFixturesDir()
+
+	copy := func(inpath string) {
+		buf, err := ioutil.ReadFile(inpath)
+		assertNoError(err, t, fmt.Sprintf("Reading %q", inpath))
+		outpath := filepath.Join(dir, "testfnpos.go")
+		assertNoError(ioutil.WriteFile(outpath, buf, 0666), t, fmt.Sprintf("Creating %q", outpath))
+	}
+
+	copy(filepath.Join(dir, "testfnpos1.go"))
+
+	withTestClient2Extended("testfnpos", t, 0, [3]string{}, nil, func(c service.Client, f protest.Fixture) {
+		_, err := c.CreateBreakpoint(&api.Breakpoint{FunctionName: "main.f1"})
+		assertNoError(err, t, "CreateBreakpoint")
+		state := <-c.Continue()
+		assertNoError(state.Err, t, "Continue")
+		t.Logf("%s:%d", state.CurrentThread.File, state.CurrentThread.Line)
+		if state.CurrentThread.Line != 5 {
+			t.Fatalf("wrong location %s:%d", state.CurrentThread.File, state.CurrentThread.Line)
+		}
+
+		// rewrite test file and restart, rebuilding
+		copy(filepath.Join(dir, "testfnpos2.go"))
+		_, err = c.Restart(true)
+		assertNoError(err, t, "Restart(true)")
+
+		state = <-c.Continue()
+		assertNoError(state.Err, t, "Continue")
+		t.Logf("%s:%d", state.CurrentThread.File, state.CurrentThread.Line)
+		if state.CurrentThread.Line != 9 {
+			t.Fatalf("wrong location %s:%d", state.CurrentThread.File, state.CurrentThread.Line)
+		}
+	})
+}
+
+func assertLine(t *testing.T, state *api.DebuggerState, file string, lineno int) {
+	t.Helper()
+	t.Logf("%s:%d", state.CurrentThread.File, state.CurrentThread.Line)
+	if !strings.HasSuffix(state.CurrentThread.File, file) || state.CurrentThread.Line != lineno {
+		t.Fatalf("wrong location %s:%d", state.CurrentThread.File, state.CurrentThread.Line)
+	}
+}
+
+func TestPluginSuspendedBreakpoint(t *testing.T) {
+	// Tests that breakpoints created in a suspended state will be enabled automatically when a plugin is loaded.
+	pluginFixtures := protest.WithPlugins(t, protest.AllNonOptimized, "plugin1/", "plugin2/")
+	dir, err := filepath.Abs(protest.FindFixturesDir())
+	assertNoError(err, t, "filepath.Abs")
+
+	withTestClient2Extended("plugintest", t, protest.AllNonOptimized, [3]string{}, []string{pluginFixtures[0].Path, pluginFixtures[1].Path}, func(c service.Client, f protest.Fixture) {
+		_, err := c.CreateBreakpointWithExpr(&api.Breakpoint{FunctionName: "github.com/go-delve/delve/_fixtures/plugin1.Fn1", Line: 1}, "", nil, true)
+		assertNoError(err, t, "CreateBreakpointWithExpr(Fn1) (suspended)")
+
+		_, err = c.CreateBreakpointWithExpr(&api.Breakpoint{File: filepath.Join(dir, "plugin2", "plugin2.go"), Line: 9}, "", nil, true)
+		assertNoError(err, t, "CreateBreakpointWithExpr(plugin2.go:9) (suspended)")
+
+		cont := func(name, file string, lineno int) {
+			t.Helper()
+			state := <-c.Continue()
+			assertNoError(state.Err, t, name)
+			assertLine(t, state, file, lineno)
+		}
+
+		cont("Continue 1", "plugintest.go", 22)
+		cont("Continue 2", "plugintest.go", 27)
+		cont("Continue 3", "plugin1.go", 6)
+		cont("Continue 4", "plugin2.go", 9)
+	})
+
+	withTestClient2Extended("plugintest", t, protest.AllNonOptimized, [3]string{}, []string{pluginFixtures[0].Path, pluginFixtures[1].Path}, func(c service.Client, f protest.Fixture) {
+		exprbreak := func(expr string) {
+			t.Helper()
+			_, err := c.CreateBreakpointWithExpr(&api.Breakpoint{}, expr, nil, true)
+			assertNoError(err, t, fmt.Sprintf("CreateBreakpointWithExpr(%s) (suspended)", expr))
+		}
+
+		cont := func(name, file string, lineno int) {
+			t.Helper()
+			state := <-c.Continue()
+			assertNoError(state.Err, t, name)
+			assertLine(t, state, file, lineno)
+		}
+
+		exprbreak("plugin1.Fn1")
+		exprbreak("plugin2.go:9")
+
+		// The following breakpoints can never be un-suspended because the
+		// expression is never resolved, but this shouldn't cause problems
+
+		exprbreak("m[0]")
+		exprbreak("*m[0]")
+		exprbreak("unknownfn")
+		exprbreak("+2")
+
+		cont("Continue 1", "plugintest.go", 22)
+		cont("Continue 2", "plugintest.go", 27)
+		cont("Continue 3", "plugin1.go", 5)
+		cont("Continue 4", "plugin2.go", 9)
+	})
+}
+
+func TestClientServer_createBreakpointWithID(t *testing.T) {
+	protest.AllowRecording(t)
+	withTestClient2("continuetestprog", t, func(c service.Client) {
+		bp, err := c.CreateBreakpoint(&api.Breakpoint{ID: 2, FunctionName: "main.main", Line: 1})
+		assertNoError(err, t, "CreateBreakpoint()")
+		if bp.ID != 2 {
+			t.Errorf("wrong ID for breakpoint %d", bp.ID)
+		}
+
+		bp2, err := c.CreateBreakpoint(&api.Breakpoint{FunctionName: "main.main", Line: 2})
+		assertNoError(err, t, "CreateBreakpoint()")
+		if bp2.ID != 3 {
+			t.Errorf("wrong ID for breakpoint %d", bp2.ID)
+		}
+	})
+}
